import React, { Suspense } from 'react';
import ReactDOM from 'react-dom/client';
import './index.css';
import App from './App';
import { ThemeProvider } from "./providers/ThemeProvider";
import { MobileProvider } from "./providers/MobileProvider";
import { AuthProvider } from "./contexts/AuthContext";
import { Loader } from "@/components/ui/loader";
import { initializePerformanceOptimizations, applyMobileOptimizations } from './lib/performance-optimizations';
import { QueryClientProvider } from '@tanstack/react-query';
import { queryClient } from './lib/queryClient';
import { Toaster } from "@/components/ui/toaster";
import { HybridConnectionProvider } from './providers/HybridConnectionProvider';
import { LanguageProvider } from './contexts/LanguageContext';
import * as iOS18Detector from './lib/ios18Detector';

// Import the real IOSRedirect component with named import
import { IOSRedirect } from './components/IOSRedirect';

// Apply performance optimizations immediately
initializePerformanceOptimizations();

// Apply mobile-specific optimizations if on a mobile device
if (/Android|webOS|iPhone|iPad|iPod|BlackBerry|IEMobile|Opera Mini/i.test(navigator.userAgent) ||
    (navigator.userAgent.includes("Mac") && "ontouchend" in document) ||
    window.innerWidth < 768) {
  applyMobileOptimizations();
  
  // Initialize iOS-specific patches immediately on detection
  try {
    // Run iOS detection using our new utility functions
    const isIOSDevice = iOS18Detector.isIOS();
    const isIOS18Device = iOS18Detector.isIOS18();
    const isIOS1831Device = iOS18Detector.isIOS1831();
    
    if (isIOSDevice) {
      // Get iOS version details for logging
      const iosVersion = iOS18Detector.getIOSVersion();
      const versionString = iosVersion 
        ? `${iosVersion.major}.${iosVersion.minor}.${iosVersion.patch}` 
        : 'unknown';
      
      console.log(`iOS device detected (${versionString}), applying specialized handling`);
      
      // Set global flag for iOS detection
      (window as any).__IOS_DETECTED = true;
      
      // For ALL iOS devices, apply compatibility fixes
      iOS18Detector.applyIOS18Fixes();
      
      // Apply iOS 18.3.1 special patches with additional flags
      if (isIOS1831Device) {
        console.log("iOS 18.3.1 detected - initializing special compatibility mode");
        
        // Set global flags for special handling in other components
        (window as any).__IOS_18_3_1_DETECTED = true;
        (window as any).__FORCE_FALLBACK_MODE = true;
        
        // Set cookie to mark iOS 18.3.1 detection for server-side awareness
        document.cookie = "iOS18_3_1_detected=true; path=/; max-age=86400";
        
        // Force the hybrid connection to use fallback mode
        localStorage.setItem('peerpa.connectionMethod', 'fallback');
      }
    }
  } catch (error) {
    // Don't prevent app start if iOS detection fails
    console.error("Error during iOS detection:", error);
  }
}

// Declare global performance metrics
declare global {
  interface Window {
    __PEERPA_LOAD_METRICS: {
      [key: string]: number | undefined;
      initStart?: number;
      appImport?: number;
      appRender?: number;
      fullAppLoaded?: number;
    };
  }
}

// Initialize the metrics object
window.__PEERPA_LOAD_METRICS = {
  initStart: performance.now()
};
const startTime = performance.now();

/**
 * Root component with all required providers
 * Fast-loading implementation to avoid timeouts
 */
function Root() {
  // Record metrics
  const renderStart = performance.now();
  window.__PEERPA_LOAD_METRICS.appImport = renderStart - startTime;
  
  // IMPORTANT: Do NOT replace the entire body as it contains our loading screen!
  // Instead, we'll just check if the loading-container exists
  const loadingContainer = document.getElementById('loading-container');
  if (!loadingContainer) {
    console.error('No loading-container found in index.html, using fallback');
    // Only create a fallback if the loading container doesn't exist
    const fallbackLoader = document.createElement('div');
    fallbackLoader.id = 'fallback-loader';
    fallbackLoader.innerHTML = `
      <div style="display:flex;align-items:center;justify-content:center;height:100vh;width:100vw;">
        <div style="text-align:center;">
          <div style="border:4px solid rgba(59,130,246,0.3);border-radius:50%;border-top:4px solid #3b82f6;width:40px;height:40px;margin:0 auto;animation:spin 1s linear infinite;"></div>
          <p style="margin-top:20px;font-family:system-ui,-apple-system,sans-serif;color:#3b82f6;">Loading PeerPa...</p>
        </div>
      </div>
      <style>@keyframes spin{0%{transform:rotate(0deg)}100%{transform:rotate(360deg)}}</style>
    `;
    document.body.appendChild(fallbackLoader);
  }
  
  // Log useful startup information
  console.log("PeerPa starting in optimized fast-loading mode");
  console.log("Device details:", {
    userAgent: navigator.userAgent,
    viewportWidth: window.innerWidth,
    viewportHeight: window.innerHeight,
    pixelRatio: window.devicePixelRatio || 1
  });
  
  // Use a streamlined implementation to render the App faster
  return (
    <QueryClientProvider client={queryClient}>
      <AuthProvider>
        <MobileProvider>
          <ThemeProvider>
            <LanguageProvider>
              <HybridConnectionProvider
                autoConnect={true}
                debug={false}
                reconnectInterval={5000}
                maxReconnectAttempts={5}
              >
                <IOSRedirect>
                  <App />
                </IOSRedirect>
                <Toaster />
              </HybridConnectionProvider>
            </LanguageProvider>
          </ThemeProvider>
        </MobileProvider>
      </AuthProvider>
    </QueryClientProvider>
  );
}

// Detect if we're on a mobile device for timeout adjustment
const isMobileDevice = /Android|webOS|iPhone|iPad|iPod|BlackBerry|IEMobile|Opera Mini/i.test(navigator.userAgent) ||
    (navigator.userAgent.includes("Mac") && "ontouchend" in document) ||
    window.innerWidth < 768;

// Use a longer timeout for mobile devices
const startupTimeout = isMobileDevice ? 15000 : 8000; // 15 seconds for mobile, 8 seconds for desktop
console.log(`Setting startup timeout to ${startupTimeout}ms (${isMobileDevice ? 'mobile' : 'desktop'} device)`);

// Create a timeout to detect and handle startup failures
let startupTimer: number | null = window.setTimeout(() => {
  console.error(`Application failed to fully initialize within ${startupTimeout}ms`);
  
  // For Android devices, we'll do one automatic retry after timeout
  const isAndroid = /Android/i.test(navigator.userAgent);
  if (isAndroid && !window.__ANDROID_RETRY_ATTEMPTED) {
    console.log('Android device detected, performing automatic retry...');
    window.__ANDROID_RETRY_ATTEMPTED = true;
    
    // Clear localStorage and sessionStorage
    try {
      localStorage.removeItem('peerpa.connectionMethod');
      localStorage.removeItem('peerpa.lastConnectionAttempt');
      sessionStorage.clear();
      console.log('Cleared storage for retry');
    } catch (e) {
      console.error('Failed to clear storage:', e);
    }
    
    // Reload with force cachebust
    window.location.href = window.location.href.split('?')[0] + '?retry=' + Date.now();
    return;
  }
  
  // Update the loading indicator to show the error
  const rootElement = document.getElementById('root');
  if (rootElement) {
    rootElement.innerHTML = `
      <div style="display:flex;align-items:center;justify-content:center;height:100vh;width:100vw;">
        <div style="text-align:center;max-width:80%;">
          <div style="width:50px;height:50px;margin:0 auto;color:#ef4444;">
            <svg xmlns="http://www.w3.org/2000/svg" fill="none" viewBox="0 0 24 24" stroke="currentColor">
              <path stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="M12 9v2m0 4h.01m-6.938 4h13.856c1.54 0 2.502-1.667 1.732-3L13.732 4c-.77-1.333-2.694-1.333-3.464 0L3.34 16c-.77 1.333.192 3 1.732 3z" />
            </svg>
          </div>
          <h2 style="margin-top:20px;font-family:system-ui,-apple-system,sans-serif;color:#111;">Startup Issue Detected</h2>
          <p style="margin-top:10px;font-family:system-ui,-apple-system,sans-serif;color:#444;">
            The application is taking longer than expected to start. This could be due to network connectivity 
            or device compatibility issues.
          </p>
          <button onclick="window.location.reload()" style="margin-top:20px;background:#3b82f6;color:white;border:none;padding:8px 16px;border-radius:4px;font-family:system-ui,-apple-system,sans-serif;cursor:pointer;">
            Retry Connection
          </button>
        </div>
      </div>
    `;
  }
}, startupTimeout);

// Function to clear the timeout when app successfully loads
window.clearStartupTimeout = () => {
  if (startupTimer !== null) {
    window.clearTimeout(startupTimer);
    startupTimer = null;
    console.log('Application started successfully, cleared startup timeout');
  }
};

// Import application initialization service
import { initializeApp } from './services/AppInitializationService';
import { initializeAndroidFixes } from './lib/androidSpecificFixes';

// Mount the application
const rootElement = document.getElementById('root');
if (!rootElement) throw new Error('Root element not found');

// Add warning handler for unhandled promise rejections
window.addEventListener('unhandledrejection', (event) => {
  console.error('Unhandled Promise Rejection:', event.reason);
});

// Initialize app and then render
// This helps with more reliable startup on problematic devices
(async () => {
  try {
    // For Android devices, we'll initialize the app first
    const isAndroid = /Android/i.test(navigator.userAgent);
    
    if (isAndroid) {
      console.log('Android device detected, initializing app before rendering...');
      
      // Show early message to user so they know something is happening
      const loadingText = document.getElementById('loading-text');
      if (loadingText) {
        loadingText.textContent = 'Preparing application...';
      }
      
      try {
        // EMERGENCY FIX: Add a loop protection flag in case of reload loops
        const loopProtection = sessionStorage.getItem('android_init_attempt');
        if (loopProtection) {
          console.log('Previous Android initialization detected, skipping to prevent loop');
        } else {
          // Set protection flag
          sessionStorage.setItem('android_init_attempt', 'true');
          
          // Apply Android-specific fixes
          console.log('Applying Android-specific fixes...');
          // This must be called first to fix critical rendering and connection issues
          initializeAndroidFixes();
          
          // Then run general app initialization
          await initializeApp();
          console.log('Android initialization complete, rendering app...');
        }
      } catch (error) {
        console.error('Android initialization failed:', error);
        // Continue with rendering anyway
      }
    }
    
    // Render the application
    ReactDOM.createRoot(rootElement).render(<Root />);
  } catch (error: any) {
    console.error('Critical rendering error:', error);
    
    // Display error message to user in case of catastrophic failure
    rootElement.innerHTML = `
      <div style="padding: 20px; text-align: center;">
        <h2>Application Error</h2>
        <p>The application encountered a critical error during startup.</p>
        <button onclick="window.location.reload()">Retry</button>
        <p><small>Error: ${error?.message || 'Unknown error'}</small></p>
      </div>
    `;
  }
})();

// Add a global declaration for the clearStartupTimeout function and Android retry flag
declare global {
  interface Window {
    clearStartupTimeout: () => void;
    __ANDROID_RETRY_ATTEMPTED?: boolean;
  }
}